from dataclasses import dataclass

@dataclass
class Config:
    '''POCO class for config data'''

    Workspace: str = ""
    Workflow: str = ""
    Baseflow: str = ""
    Backup: str = ""
    Selection: str = ""
    Edited: str = ""
    Preview: str = ""
    Editing: str = ""
    Instagram: str = ""

    def __str__(self):
        '''Overide the __str__ class
        
        Returns:
            (string): Returns the overrode string
        '''
        
        return f"""
                Workspace: {self.Workspace}
                Workflow: {self.Workflow}
                Baseflow: {self.Baseflow}
                Backup: {self.Backup}
                Selection: {self.Selection}
                Edited: {self.Edited}
                Preview: {self.Preview}
                Editing: {self.Editing}
                Instagram: {self.Instagram}
                """

@dataclass
class Context:
    '''POCO class for context data'''
    
    Config: str = ""
    WorkspaceObj: object = None



import sys
import os
import json
import shutil
import subprocess
import click
import picturebot as pb
from picturebot.helper import Helper as helper
import picturebot.poco as poco
import generalutils.guard as grd
import picturebot.workspace as ws
from picturebot.directory import Directory as directory
import picturebot.base as baseflow
import picturebot.flow as otherflow

class Flow:
    def __init__(self, context):
        self.ctx = context
    
    def Edited(self):
        # Get the current working directory of where the script is executed
        cwd = os.getcwd()

        # Check whether the current working directory exists
        grd.Filesystem.PathExist(cwd)

        # Obtain the name of the base directory of the current working directory
        shoot = os.path.basename(cwd)
    
        # Obtain the path to the base flow project
        pathToBaseflowProject = helper.FullFilePath(self.ctx.Config.Workspace, self.ctx.Config.Selection, shoot)

        # Check whether the the path to the base flow project exists
        grd.Filesystem.PathExist(pathToBaseflowProject)

        # Check whether you're within the baseflow directory
        grd.Filesystem.PathCwdExists(pathToBaseflowProject, cwd, True)

        # Obtain the absolute path to the edit flow of a shoot
        pathToShootInEditedFlow = helper.FullFilePath(self.ctx.Config.Workspace, self.ctx.Config.Edited, shoot)

        # Check whether you're within the edit flow shoot directory
        grd.Filesystem.PathExist(pathToShootInEditedFlow)

        # Open the correct edited directory
        os.system(f"explorer {pathToShootInEditedFlow}") 

    def Completed(self):
        # Get the current working directory of where the script is executed
        cwd = os.getcwd()

        # Check whether the current working directory exists
        grd.Filesystem.PathExist(cwd)
        
        # Check whether the script is executed from the workspace directory
        grd.Filesystem.PathCwdExists(self.ctx.Config.Workspace, cwd)

        # Obtain the path to the the edit root directory
        pathToEditedRoot = helper.FullFilePath(self.ctx.Config.Workspace, self.ctx.Config.Edited)

        # Check whether the path to the edit root directory exists
        grd.Filesystem.PathExist(pathToEditedRoot)

        # Loopover every shoot within the edit directory
        for shoot in os.listdir(pathToEditedRoot):
            # Obtain the path to the shoot within the edit root directory
            pathToEditedShoots = helper.FullFilePath(pathToEditedRoot, shoot)

            # Check whether the shoot exists within the edit root directory
            grd.Filesystem.PathExist(pathToEditedShoots)

            # Amount of pictures within the edit shoot directory
            shootAmountPicturesEdited = len(os.listdir(pathToEditedShoots))

            pathToSelectedShoot = helper.FullFilePath(self.ctx.Config.Workspace, self.ctx.Config.Selection, shoot)
            grd.Filesystem.PathExist( pathToSelectedShoot)

            # Amount of pictures within the selection shoot directory
            shootAmountPicturesSelection = len(os.listdir(pathToSelectedShoot))

            # Check whether the amount of pictures are equal
            if shootAmountPicturesEdited != shootAmountPicturesSelection:
                print(f'Shoot: {shoot} is not fully edited')



import sys
import os
import json
import shutil
import subprocess
import click
import picturebot as pb
from picturebot.helper import Helper as helper
import picturebot.poco as poco
import generalutils.guard as grd
import picturebot.workspace as ws
from picturebot.directory import Directory as directory
import re

class Base():
    def __init__(self, ctx, index):
        '''Base class constructor
        
        Args:
            ctx (object): Global context object
            index (int): Index of the selected workspace configuration
        '''

        self.ctx = ctx
        self.index = int(index)
        self.cwd = ""
        self.shoot = ""
    
    def Backup(self, path):
        '''Backup a single picture in the backup flow
        
        Args:
            path (string): Path to the picture
        '''

        # Check whether the script is runned from the base flow directory
        self.__PathToBaseFlow()
        # Copy a single file in the backup flow
        self.__CopyFile(path)

    def MassBackup(self):
        '''Backup all the pictures within the base flow in the backup flow'''
        # Check whether the script is runned from the base flow directory
        self.__PathToBaseFlow()
        # Copy all pictures within the base flow in the backup flow
        self.__CopyFilesFromBaseToBackup()

    def Rename(self, index, path):
        '''Rename a file accordingly to the shootname (<name>_<data>_<index>)
        
        Args:
            index (string): Picture index number
            path (string): Path to the picture
        '''

        # Check whether the script is runned from the base flow directory
        self.__PathToBaseFlow()
        shoot = self.NewShootName()
        self.__RenamePicture(path, shoot, index)

    def MassRename(self):
        '''Rename all the files within a flow'''

        # Check whether the script is runned from the base flow directory
        self.__PathToBaseFlow()
        self.__Rename()

    def HashRename(self, index, path):
        '''Method which renames filenames with their hashed values
        
        Args:
            index (string): Picture index number
            path (string): Path to the picture
        '''

        self.__PathToBaseFlow()
        self.__Hashed(path, index)
    
    def Convert(self, path, quality):
        '''Convert a raw picture to a jpg format and store it within the preview flow
        
        Args:
            path (string): Path of the picture
            quality (string): Quality of the converted picture
        '''

        # Check whether the script is runned from the base flow directory
        self.__PathToBaseFlow()
        self.__ConvertImage(path, quality)

    def __ConvertImage(self, path, quality):
        '''Convert a raw picture to a jpg format and store it within the preview flow
        
        Args:
            path (string): Path of the picture
            quality (string): Quality of the converted picture
        '''

        # Obtain the basename of the path
        basename = os.path.basename(path)
        # Obtain the basename without an extension
        basenameWithoutExtension = basename.split('.')[0]
        # Create the output storage path
        output = os.path.join(self.ctx.Config[self.index].Workspace, self.shoot, self.ctx.Config[self.index].Preview, f"{basenameWithoutExtension}.jpg")
        
        # magick convert "<path>" -quality <quality>% -verbose "<outputPath>"
        command = f"magick convert \"{path}\" -quality {quality}% -verbose \"{output}\""

        # Execute command
        os.system(command)

    def __PathToBaseFlow(self):
        '''Check whether the script is runned within the baseflow directory'''

        # Get the current working directory of where the script is executed
        self.cwd = os.getcwd()

        self.shoot = re.search('(\w+( +\w+)*\s+\d+-\d+-\d+)', self.cwd).group(0)

        # Check whether the current working directory exists
        grd.Filesystem.PathExist(self.cwd)

        # Obtain the path to the base flow project
        if grd.Filesystem.IsPathCwd(self.ctx.Config[self.index].Baseflow, self.cwd):
            pathToBaseflowProject = helper.FullFilePath(self.ctx.Config[self.index].Workspace, self.shoot, self.ctx.Config[self.index].Baseflow)
            # Check whether the the path to the base flow project exists
            grd.Filesystem.PathExist(pathToBaseflowProject)
            # Check whether you're within the backup flow directory
            grd.Filesystem.PathCwdExists(pathToBaseflowProject, self.cwd, True)
        
    def __CopyFilesFromBaseToBackup(self):
        '''Copy all files from the base flow to the backup flow'''

        pictures = os.listdir(self.cwd)

        counter = 0

        for picture in pictures:
            pathToBackupFlow = self.__CopyFile(picture)

            click.echo(f'Copying: {picture} -> {pathToBackupFlow} [{counter + 1}/{len(pictures)}]')

            counter += 1

        click.echo(f"Copied files: {counter}")

    def __CopyFile(self, path):
        '''Copy a file to the backup flow
        
        Args:
            path (string): Path to the picture
        
        Returns:
            (string): Backup flow path
        '''

        # Obtain the filename from a specified path
        filename = os.path.basename(path)

        # Obtain the path to the baseflow directory
        pathToBaseflow = helper.FullFilePath(self.ctx.Config[self.index].Workspace, self.shoot, self.ctx.Config[self.index].Baseflow)
        # Check whether the baseflow directory exists
        grd.Filesystem.PathExist(pathToBaseflow)

        # Obtain the path the source picture within the baseflow directory
        pathToPictureSource = helper.FullFilePath(pathToBaseflow, filename)
        # Check whether the source picture path exists
        grd.Filesystem.PathExist(pathToPictureSource)

        # Obtain the path to the backupflow directory
        pathToBackupFlow = helper.FullFilePath(self.ctx.Config[self.index].Workspace, self.shoot, self.ctx.Config[self.index].Backup)
        # Check whether the backupflow directory exists
        grd.Filesystem.PathExist(pathToBackupFlow)

        # Obtain the full path name to the picture's destition path
        pathToPictureDestination = helper.FullFilePath(pathToBackupFlow, filename)
        
        # Copying picture from source to destination including the metadata
        shutil.copy2(pathToPictureSource, pathToPictureDestination)

        # Check whether the file is successfully copied
        grd.Filesystem.PathExist(pathToPictureDestination)

        return pathToBackupFlow

    def __Rename(self):
        '''Rename the picture'''

        counter = 0

        shoot = self.NewShootName()

        # Obtain the original picture name within a flow directory
        pictures = os.listdir(self.cwd)
        # sort by date
        pictures.sort(key=os.path.getctime)

        count = len(pictures)

        # Loop over every picture withing the flow directory
        for index, picture in enumerate(pictures, 1):
            self.__RenamePicture(picture, shoot, index)
        click.echo(f"Renamed files: {count}")

    def NewShootName(self):
        '''Create the shootname
        
        Returns:
            (string): Returns the formated shootname
        '''

        newShoot = ''

        # Loop over every word of the flow name directory
        for i in self.shoot.split(' '):
            # Append the individual words to an '_'
            newShoot += f'{i}_'
        return newShoot

    def __RenamePicture(self, path, shoot, index):
        '''Rename a picture
        
        Args:
            path (string): Path to the picture
            shoot (string): Shootname of the picture
            index (string): Picture index number
        '''

        # Get the extension of the original picture
        extension = path.split('.')[1]

        # Get absolute path to the picture
        pathToPicture = os.path.join(self.cwd, path)

        # Check whether the absolute path to the picture is existing
        grd.Filesystem.PathExist(path)

        # Get the new name for the picture
        newName = f"{shoot}{str(index).zfill(5)}.{extension}"

        # Obtain the absolute path to the new picture name
        pathToNewPicture = os.path.join(self.cwd, newName)
        
        # Only rename the changed files
        if not pathToNewPicture == pathToPicture:
            # Rename the picture file
            os.rename(pathToPicture, pathToNewPicture)

            # Check whether the new picture file exists after renaming
            grd.Filesystem.PathExist(pathToNewPicture)

    def __Hashed(self, path, index):
        '''Method which renames filenames with their hashed values
        
        Args:
            index (string): Picture index number
            path (string): Path to the picture
        '''
        
        extension = path.split('.')[1]

        # Get absolute path to the picture
        pathToPicture = os.path.join(self.cwd, path)

        # Check whether the absolute path to the picture is existing
        grd.Filesystem.PathExist(path)

        md5Hash = helper.HashFileMd5(path)

        # Get the new name for the picture, obtain the first 10 hashvalues
        hashedName = f"pb_{md5Hash[:10]}_{str(index).zfill(5)}.{extension}"
        
        # Obtain the absolute path to the new picture name
        pathToNewPicture = os.path.join(self.cwd, hashedName)
        
        # Only rename the changed files
        if not pathToNewPicture == pathToPicture:
            # Rename the picture file
            os.rename(pathToPicture, pathToNewPicture)

            # Check whether the new picture file exists after renaming
            grd.Filesystem.PathExist(pathToNewPicture)



import os
import sys
import hashlib
import generalutils.guard as grd
import picturebot.poco as poco

class Helper:
    '''Static class containing helper methods'''

    @staticmethod
    def FullFilePath(*items):
        '''Get the full path to a folder or directory
        Args:
            items (list): List of folders and subfolders
        
        Returns:
            (string) Full file name
        '''

        # Get the current working directory
        path = os.path.dirname(os.path.abspath(__file__))

        # Loop over every folder and subfolders
        for item in items:
            path = os.path.join(path, item)

        return path

    @staticmethod
    def HashFileMd5(file):
        '''Create a md5 hash from a file

        Args:
            file (file): File where a hash is created from
        Returns:
            (string) Md5 hash
        '''
        
        bufSize = 32768 # Read file in 32kb chunks
        md5 = hashlib.md5()

        with open(file, 'rb') as f:
            while True:
                data = f.read(bufSize)

                if not data:
                    break

                md5.update(data)
                
            return md5.hexdigest()

    @staticmethod
    def Context(context):
        return poco.Context(context.obj['config'], context.obj['workspaceObj'])



"""Console script for picturebot."""

import sys
import os
import json
import shutil
import subprocess
import click
import picturebot as pb
from picturebot.helper import Helper as helper
import picturebot.poco as poco
import generalutils.guard as grd
import picturebot.workspace as ws
from picturebot.directory import Directory as directory
import picturebot.base as baseflow
import picturebot.flow as otherflow
import picturebot.shoot as sht

@click.group()
@click.pass_context
def main(context):
    '''Main method where config data and workspace object are initialized
    
    Args:
        context (object): Global context object
    '''

    pathToConfig = helper.FullFilePath("config.json")

    # Check whether the path to the confile exists
    grd.Filesystem.PathExist(pathToConfig)

    with open(pathToConfig) as f:
        # if conext isn't initialized created a new dictionary
        if context.obj is None:
            context.obj = dict()

         # Load data from file
        data = json.load(f)
        lstConfig = []
        for d in data:
            lstConfig.append(poco.Config(d['workspace'], d['workflow'], d['baseflow'], d['backup'], d['selection'], d['edited'], d['preview'], d['editing'], d['instagram']))
        c = 1

        # Load the config data in the context variable
        #x = poco.Config(data['workspace'], data['workflow'], data['baseflow'], data['backup'], data['selection'], data['edited'], data['preview'], data['editing'], data['instagram'])
        #context.obj['config'] = poco.Config(data[c]['workspace'], data[c]['workflow'], data[c]['baseflow'], data[c]['backup'], data[c]['selection'], data[c]['edited'], data[c]['preview'], data[c]['editing'], data[c]['instagram'])
        context.obj['config'] = lstConfig
        # Load the workspace object into the context variable
        context.obj['workspaceObj'] = ws.Workspace(pathToConfig, context)

        # print(data[1]['workspace'])
        # print(context.obj['config'])

@main.command()
@click.option('--create', '-c', nargs=1, help='Create a new workspace')
@click.pass_context
def workspace(context, create):
    '''Create a new workspace
    
    Args:
        context (object): Global context object
        create (object): Create a new workspace
    '''

    ctx = helper.Context(context)
    # Get the current working directory of where the script is executed
    cwd = os.getcwd()

    #Check whether the current working directory exists
    grd.Filesystem.PathExist(cwd)

    if create:
        ctx.WorkspaceObj.Create(create[0])   

@main.command()
@click.option('--backup', '-b', nargs=2, type=str, help='Make a copy of a picture in the backup flow')
@click.option('--massbackup', '-mb', nargs=1, type=str, help='Make a copy of all pictures within the base flow and copy them to the backup flow')
@click.option('--rename', '-r', nargs=3, type=str, help='Rename a picture within the baseflow accordingly to it\'s shootname')
@click.option('--hash', '-h', nargs=3, type=str, help='Hash rename a picture')
@click.option('--massrename', '-mr', nargs=1, help='Rename all pictures within the baseflow accordingly to it\'s shootname')
@click.option('--convert', '-c', nargs=3, type=str, help='Convert a raw picture within the base flow to a jpg format and store it within the preview flow')
@click.pass_context
def base(context, backup, massbackup, hash, rename, massrename, convert):
    '''Method to backup files from the baseflow project

    Args:
        context (object): Global context object
        backup (object): Make a copy of a picture in the backup flow
        massbackup (object): Make a copy of all pictures within the base flow and copy them to the backup flow
        rename (object): Rename a picture within the baseflow accordingly to it's shootname
        massrename (object): Rename all pictures within the baseflow accordingly to it's shootname
        convert (object): Convert a raw picture within the base flow to a jpg format and store it within the preview flow
    '''

    ctx = helper.Context(context)

    if backup:
        bs = baseflow.Base(ctx, backup[0])
        bs.Backup(backup[1])
    elif massbackup:
        bs = baseflow.Base(ctx, massbackup[0])
        bs.MassBackup()
    elif hash:
        bs = baseflow.Base(ctx, hash[0])
        bs.HashRename(hash[1], hash[2])
    elif rename:
        bs = baseflow.Base(ctx, rename[0])
        bs.Rename(rename[1], rename[2])
    elif massrename:
        bs = baseflow.Base(ctx, massrename[0])
        bs.MassRename()
    elif convert:
        bs = baseflow.Base(ctx, convert[0])
        bs.Convert(convert[1], convert[2])

@main.command()
@click.option('--show', '-s', is_flag=True, help='Open config file in an editor')
@click.option('--location', '-l', is_flag=True, help='Print config file location')
@click.option('--version', '-v', is_flag=True, help='Print picturebot script version')
@click.pass_context
def config(context, show, location, version):
    '''CLI command that handles the configuration file operations
    
    Args:
        context (object): Global context object
        view (object): Option that opens the configuration file
        location (object): Option that prints the configuration file location within the filesystem
    '''

    ctx = helper.Context(context)

    if show:
        ctx.WorkspaceObj.ShowConfig()
    elif location:
        ctx.WorkspaceObj.PrintConfig()
    elif version:
        ctx.WorkspaceObj.Version()

@main.command()
@click.option('--completed', '-c', is_flag=True, help='View config file')
@click.option('--edited', '-e', is_flag=True, help='View config file')
@click.pass_context
def flow(context, completed, edited):
    ctx = helper.Context(context)

    fw = otherflow.Flow(ctx)
    if completed:
        fw.Completed()
    elif edited:
        fw.Edited()

@main.command()
@click.option('--new', '-n', nargs=3, type=str, help='Create a new shoot')
@click.pass_context
def shoot(context, new):
    '''Shoot option allows modification of a shoot within the workspace

    Args:
        context (object): Global context object
        new (object): Option to create a new shoot (<name> <date>) 
    '''

    ctx = helper.Context(context)

    if new:
        newShoot = f'{new[1]} {new[2]}'
        # Create a shoot object
        s = sht.Shoot(ctx, new[0], newShoot)
        # Creates the shoot
        s.Create()

if __name__ == "__main__":
    main() # pragma: no cover
